searchNodes=[{"doc":"","ref":"PolymorphicEmbed.html","title":"PolymorphicEmbed","type":"behaviour"},{"doc":"","ref":"PolymorphicEmbed.html#c:get_polymorphic_module/1","title":"PolymorphicEmbed.get_polymorphic_module/1","type":"callback"},{"doc":"","ref":"PolymorphicEmbed.html#c:get_polymorphic_type/1","title":"PolymorphicEmbed.get_polymorphic_type/1","type":"callback"},{"doc":"","ref":"PolymorphicEmbed.HTML.Form.html","title":"PolymorphicEmbed.HTML.Form","type":"module"},{"doc":"","ref":"PolymorphicEmbed.HTML.Form.html#polymorphic_embed_inputs_for/4","title":"PolymorphicEmbed.HTML.Form.polymorphic_embed_inputs_for/4","type":"function"},{"doc":"","ref":"PolymorphicEmbed.HTML.Form.html#to_form/5","title":"PolymorphicEmbed.HTML.Form.to_form/5","type":"function"},{"doc":"Polymorphic embeds for Ectopolymorphic_embed brings support for polymorphic/dynamic embedded schemas in Ecto.Ecto's embeds_one macro requires a specific schema module to be specified. This library removes this restriction by dynamically determining which schema to use, based on data to be stored (from a form or API) and retrieved (from the data source).","ref":"readme.html","title":"Polymorphic embeds for Ecto","type":"extras"},{"doc":"Enable polymorphismLet's say we want a schema Reminder representing a reminder for an event, that can be sent either by email or SMS.We create the Email and SMS embedded schemas containing the fields that are specific for each of those communication channels.The Reminder schema can then contain a :channel field that will either hold an Email or SMS struct, by setting its type to the custom type that this library provides.Find the schema code and explanations below.defmodule MyApp.Reminder do use Ecto.Schema import Ecto.Changeset schema &quot;reminders&quot; do field :date, :utc_datetime field :text, :string field :channel, MyApp.ChannelData end def changeset(struct, values) do struct |&gt; cast(values, [:date, :text, :channel]) |&gt; validate_required(:date) end enddefmodule MyApp.ChannelData do use PolymorphicEmbed, types: [ sms: MyApp.Channel.SMS, email: [module: MyApp.Channel.Email, identify_by_fields: [:address, :confirmed]] ] enddefmodule MyApp.Channel.Email do use Ecto.Schema import Ecto.Changeset @primary_key false embedded_schema do field :address, :string field :confirmed, :boolean end def changeset(email, params) do email |&gt; cast(params, ~w(address confirmed)a) |&gt; validate_required(:address) |&gt; validate_length(:address, min: 4) end enddefmodule MyApp.Channel.SMS do use Ecto.Schema @primary_key false embedded_schema do field :number, :string end endYou have noticed in the code above that you need to define an intermediary module, in this example ChannelData. This module is the Ecto Type (through use-ing PolymorphicEmbed).The :types option for PolymorphicEmbed contains a keyword list mapping an atom representing the type (in this example :email and :sms) with the corresponding embedded schema module.There are two strategies to detect the right embedded schema to use:1.[sms: MyApp.Channel.SMS]When receiving parameters to be casted (e.g. from a form), we expect a &quot;__type__&quot; (or :__type__) parameter containing the type of channel (&quot;email&quot; or &quot;sms&quot;).2.[email: [ module: MyApp.Channel.Email, identify_by_fields: [:address, :confirmed]]]Here we specify how the type can be determined based on the presence of given fields. In this example, if the data contains :address and :confirmed parameters (or their string version), the type is :email. A &quot;__type__&quot; parameter is then no longer required.Note that you may still include a __type__ parameter that will take precedence over this strategy (this could still be useful if you need to store incomplete data, which might not allow identifying the type).Displaying form inputs and errors in Phoenix templatesThe library comes with a form helper in order to build form inputs for polymorphic embeds and display changeset errors.In the entrypoint defining your web interface (lib/your_app_web.ex file), add the following import:def view do quote do # imports and stuff import PolymorphicEmbed.HTML.Form end endThis provides you with a polymorphic_embed_inputs_for/4 function.Here is an example form using the imported function:&lt;%= inputs_for f, :reminders, fn reminder_form -&gt; %&gt; &lt;%= polymorphic_embed_inputs_for reminder_form, :channel, :sms, fn sms_form -&gt; %&gt; &lt;div class=&quot;sms-inputs&quot;&gt; &lt;label&gt;Number&lt;label&gt; &lt;%= text_input sms_form, :number %&gt; &lt;div class=&quot;error&quot;&gt; &lt;%= error_tag sms_form, :number %&gt; &lt;/div&gt; &lt;/div&gt; &lt;% end %&gt; &lt;% end %&gt;polymorphic_embed_inputs_for/4 also renders a hidden input for the &quot;__type__&quot; field.","ref":"readme.html#usage","title":"Polymorphic embeds for Ecto - Usage","type":"extras"},{"doc":"Detect which types to use for the data being cast-ed, based on fields present in the data (no need for a type field in the data)Run changeset validations when a changeset/2 function is present (when absent, the library will introspect the fields to cast)Support for nested polymorphic embedsSupport for nested embeds_one/embeds_many embedsDisplay form inputs for polymorphic embeds in Phoenix templatesTests to ensure code quality","ref":"readme.html#features","title":"Polymorphic embeds for Ecto - Features","type":"extras"},{"doc":"Add polymorphic_embed for Elixir as a dependency in your mix.exs file:def deps do [ {:polymorphic_embed, &quot;~&gt; 0.7.0&quot;} ] end","ref":"readme.html#installation","title":"Polymorphic embeds for Ecto - Installation","type":"extras"},{"doc":"HexDocs documentation can be found at https://hexdocs.pm/polymorphic_embed.","ref":"readme.html#hexdocs","title":"Polymorphic embeds for Ecto - HexDocs","type":"extras"}]